from CPU import *


def xchg(self,cpu):
    result = cpu.x
    cpu.x = cpu.y
    cpu.y = result

xchg_instr = Instr(0x00, "xchg", "exchange x and y")
xchg_instr.operation = xchg

def AND(self,cpu):
    result = (cpu.x & cpu.y)
    cpu.shift(1,0)
    cpu.x = result

and_instr = Instr(0x10, "and", "bitwise AND x and y, shift left, store result in x")
and_instr.operation = AND

def OR(self,cpu):
    result = (cpu.x | cpu.y)
    cpu.shift(1,0)
    cpu.x = result

or_instr = Instr(0x20, "or", "bitwise OR x and y, shift left, store result in x")
or_instr.operation = OR


def xor(self,cpu):
    result = (cpu.x ^ cpu.y)
    cpu.shift(1,0)
    cpu.x = result

xor_instr = Instr(0x30, "xor", "bitwise XOR x and y, shift left, store result in x")
xor_instr.operation = xor


def add(self,cpu):
    result = (cpu.x + cpu.y) & 0xffff
    cpu.shift(1,0)
    cpu.x = result

add_instr = Instr(0x40, "add", "add x and y, shift left, store the result in x")
add_instr.operation = add


def mul(self,cpu):
    result = (cpu.x * cpu.y) & 0xffff
    cpu.shift(1,0)
    cpu.x = result

mul_instr = Instr(0x50, "mul", "multiply x and y, shift left, store result in x")
mul_instr.operation = mul

def pop(self,cpu):
    result = cpu.x
    cpu.shift(1,0)
    cpu.shift(0,1)
    cpu.s = result

pop_instr = Instr(0x60, "pop", "pop x from staqe, push it to stashe")
pop_instr.operation = pop

def disc(self,cpu):
    cpu.shift(1,0)

disc_instr = Instr(0x70, "disc", "pop X from staqe, discard it")
disc_instr.operation = disc

def pushl(self,cpu):
    if(cpu.read_c == 0):
        cpu.shift(0,0)
        cpu.x = 0
    value = (self.opcode & 0x0F) << (cpu.read_c*4)
    cpu.x = cpu.x | value

pushl_instr = Instr(0x80, "pushl", "push literal")
pushl_instr.operation = pushl

def jmp(self, cpu):
    cpu.ip = cpu.x
    shift(0,1)

jmp_instr = Instr(0x90, "jmp", "jump to location pointed to by x")
jmp_instr.operation = jmp

def sign(self, cpu):
    cpu.x = cpu.x >> 15

sign_instr = Instr(0xa0, "sign", "change x to sign bit of x")
sign_instr.operation = sign

def neg(self, cpu):
    cpu.x *= -1

neg_instr = Instr(0xb0, "neg", "arithmetic negate x")
neg_instr.operation = neg

def ld(self, cpu):
    value = cpu.mem[cpu.x]
    if(cpu.x & 1 == 0):
        value = value << 4 | cpu.mem[cpu.x+1] 
    cpu.x = value

ld_instr = Instr(0xc0, "ld", "load *y in x, increment y by 2")
ld_instr.operation = ld

def st(self, cpu):
    if(cpu.y & 1 == 1):
        cpu.mem[cpu.y] = cpu.x & 0xff
    else:
        cpu.mem[cpu.y] = cpu.x & 0xff00
        cpu.mem[cpu.y+1] = cpu.x & 0x00ff
    cpu.shift(1,0)
    cpu.x += 2

st_instr = Instr(0xd0, "st", "store x in *y")
st_instr.operation = st

def dup(self, cpu):
    cpu.shift(0,0)

dup_instr = Instr(0xe0, "dup", "push x onto the staqe")
dup_instr.operation = dup

def push(self, cpu):
    cpu.shift(0,0)
    cpu.x = cpu.s
    cpu.shift(1,1)
    
push_instr = Instr(0xf0, "push", "pop s from stashe, push it to staqe")
push_instr.operation = push
