module staqe
  (
   
   //////////// CLOCK //////////
   CLOCK_50,
   CLOCK2_50,
   CLOCK3_50,

   //////////// LED //////////
   LEDG,
   LEDR,

   //////////// KEY //////////
   KEY,

   //////////// SW //////////
   SW,

   //////////// SEG7 //////////
   HEX0,
   HEX1,
   HEX2,
   HEX3,
   HEX4,
   HEX5,
   HEX6,
   HEX7,

   //////////// PS2 for Keyboard and Mouse //////////
   PS2_CLK,
   PS2_CLK2,
   PS2_DAT,
   PS2_DAT2,

   //////////// SDRAM //////////
   DRAM_ADDR,
   DRAM_BA,
   DRAM_CAS_N,
   DRAM_CKE,
   DRAM_CLK,
   DRAM_CS_N,
   DRAM_DQ,
   DRAM_DQM,
   DRAM_RAS_N,
   DRAM_WE_N,

   //////////// SRAM //////////
   SRAM_ADDR,
   SRAM_CE_N,
   SRAM_DQ,
   SRAM_LB_N,
   SRAM_OE_N,
   SRAM_UB_N,
   SRAM_WE_N 
   );

   //=======================================================
   //  PARAMETER declarations
   //=======================================================


   //=======================================================
   //  PORT declarations
   //=======================================================

   //////////// CLOCK //////////
   input 		          		CLOCK_50;
   input 		          		CLOCK2_50;
   input 		          		CLOCK3_50;

   //////////// LED //////////
   output [8:0] 				LEDG;
   output [17:0] 				LEDR;

   //////////// KEY //////////
   input [3:0] 					KEY;

   //////////// SW //////////
   input [17:0] 				SW;

   //////////// SEG7 //////////
   output [6:0] 				HEX0;
   output [6:0] 				HEX1;
   output [6:0] 				HEX2;
   output [6:0] 				HEX3;
   output [6:0] 				HEX4;
   output [6:0] 				HEX5;
   output [6:0] 				HEX6;
   output [6:0] 				HEX7;

   //////////// PS2 for Keyboard and Mouse //////////
   inout 		          		PS2_CLK;
   inout 		          		PS2_CLK2;
   inout 		          		PS2_DAT;
   inout 		          		PS2_DAT2;

   //////////// SDRAM //////////
   output [12:0] 				DRAM_ADDR;
   output [1:0] 				DRAM_BA;
   output		          		DRAM_CAS_N;
   output		          		DRAM_CKE;
   output		          		DRAM_CLK;
   output		          		DRAM_CS_N;
   inout [31:0] 				DRAM_DQ;
   output [3:0] 				DRAM_DQM;
   output		          		DRAM_RAS_N;
   output		          		DRAM_WE_N;

   //////////// SRAM //////////
   output [19:0] 				SRAM_ADDR;
   output		          		SRAM_CE_N;
   inout [15:0] 				SRAM_DQ;
   output		          		SRAM_LB_N;
   output		          		SRAM_OE_N;
   output		          		SRAM_UB_N;
   output		          		SRAM_WE_N;

   // --------

   reg 						CLK;

   always @(posedge SW[0] or negedge SW[17]) begin
      if (!SW[17]) CLK <= 'h0;
      else CLK <= 'h1;
   end

   // --------

/*   ss_display(X[7:4], HEX7[6:0]);
   ss_display(X[3:0], HEX6[6:0]);
   ss_display(Y[7:4], HEX5[6:0]);
   ss_display(Y[3:0], HEX4[6:0]);
   ss_display(DISPLAY[15:12], HEX3[6:0]);
   ss_display(DISPLAY[11:8], HEX2[6:0]);
   ss_display(DISPLAY[7:4], HEX1[6:0]);
   ss_display(DISPLAY[3:0], HEX0[6:0]);
   assign LEDG[8] = CLK;
   assign LEDG[7:0] = NX;
   assign LEDR[15:0] = ALU_OUT;
   assign LEDR[17] = L_NIBBLE_NEW;
   wire[15:0] DISPLAY = (KEY[3])?{PC[7:0],RAM_INST_OUT}:{1'h0, ALU_OP,4'h0, CONTROL_BUF[5:4],2'h0,CONTROL_BUF[3:0]};
   // --------

   reg [15:0] NX, NS, NPC;
   reg [1:0]  L_NIBBLE;	// literal nibble index counter
   wire       L_NIBBLE_NEW = (L_NIBBLE[0] && !L_NIBBLE[1]);
   wire [2:0] ALU_OP;
   wire [15:0] X, Y, S, PC;

   // --------

   ram(
       RAM_DATA_ADDR,
       RAM_INST_ADDR,
       ~CLK,
       CLK,
       RAM_DATA_IN,
       0,
       RAM_DATA_WRITE,
       0,
       RAM_DATA_OUT,
       RAM_INST_OUT);

   wire [7:0]  RAM_INST_OUT;
   wire [15:0] RAM_INST_ADDR = PC;
   wire        RAM_DATA_WRITE = (OPCODE == 4'hD);
   wire [15:0] RAM_DATA_BYTE_ADDR = RAM_DATA_WRITE? Y:X;
   wire [14:0] RAM_DATA_ADDR = RAM_DATA_BYTE_ADDR[15:1];
   wire [15:0] RAM_DATA_IN = RAM_DATA_BYTE_ADDR[0]? X:(X&8'hFF);
   wire [15:0] RAM_DATA_OUT;

   // -------

   program_counter(CLK, NPC, PC, PC_LOAD);
   stack(CLK, NX, X, Y, CONTROL[3:0]);
   control_block(OPCODE, L_NIBBLE_NEW, STASH_ENABLE, STASH_OP, ST_XCHG, ST_STICK, ST_OR, ST_RIGHT, ALU_OP);
   stash(CLK, NS, S, CONTROL[5:4]);
   wire [5:0]  CONTROL = {STASH_OP, STASH_ENABLE, ST_XCHG, ST_STICK, ST_OR, ST_RIGHT};
   wire        STASH_OP, STASH_ENABLE;
   wire        ST_XCHG, ST_STICK, ST_OR, ST_RIGHT;
   reg [5:0]   CONTROL_BUF;
   wire [3:0]  OPCODE = RAM_INST_OUT[7:4];
   wire [3:0]  INST_LITERAL = RAM_INST_OUT[3:0];
   wire        PC_LOAD = (OPCODE == 9);
   wire [15:0] ALU_OUT;

   ALU(X, Y, ALU_OP, ALU_OUT);

   // -------

   always @(posedge CLK) begin
      
      if(OPCODE == 8) begin
	 if(L_NIBBLE == 0) NX = INST_LITERAL;
	 else if(L_NIBBLE == 1) NX = (INST_LITERAL<<4);
	 else if(L_NIBBLE == 2) NX = (INST_LITERAL<<8);
	 else if(L_NIBBLE == 3) NX = (INST_LITERAL<<12);
	 L_NIBBLE = L_NIBBLE + 2'b1;
      end else begin
	 L_NIBBLE = 0;
	 if((OPCODE <= 5) ||
	    (OPCODE == 4'hA) ||
	    (OPCODE == 4'hB) ||	
	    (OPCODE == 4'hE)) NX = ALU_OUT;
	 else if(OPCODE==4'hC) NX = RAM_DATA_OUT;
	 else if(OPCODE==4'hD) NX = (Y+2'h2)&16'hFFFF;
	 else if(OPCODE==4'hF) NX = S;
	 else NX = 0;
      end
      if(OPCODE == 9) NPC = ALU_OUT;
      if(OPCODE == 6) NS = X;
      CONTROL_BUF = {STASH_OP, STASH_ENABLE, ST_XCHG, ST_STICK, ST_OR, ST_RIGHT};
   end
*/
endmodule
