// A small stack of 16 16bit registers.
// Can perform push and pop operations.
//
// IN : Input, which may be pushed on the stack.
// X  : Output, assigned to the top element of the stack.
// STASHE_OP : 0 if push, 1 if pop. Any other values implies no action.
// CLK : The clock. Shifting is to be done on the negedge.

module stash(CLK, IN, X, OP);
    input[15:0] IN;
    output[15:0] X;
	 input[1:0] OP;
    wire STASHE_OP = OP[0];
	 wire STASHE_ENABLE = OP[1];
    input CLK;

    reg[15:0] data[15:0];

    assign X = data[0];
	 integer i;
	 
    always@(negedge CLK) begin
       if(STASHE_ENABLE) begin
          if(STASHE_OP == 0) begin
            // STASHE_OP == 0 means push
            for(i=15; i>0; i = i-1) data[i]=data[i-1];
            data[0] = IN;
          end else if(STASHE_OP == 1) begin
            // STASHE_OP == 1 means pop
            for(i=0; i<15; i=i+1) data[i]=data[i+1];
            data[15] = 0;
          end
		 end
    end

endmodule